use rusqlite::{types::*, *};

const DBPATH: &'static str = "data/tracker.db";
const STATEMENTS: &'static [&str] = &[
    "CREATE TABLE log (
   id integer primary key autoincrement,
   time text,
   serial integer,
   latitude float,
   longitude float,
   speed integer,
   height integer,
   direction integer,
   is_satellite bool);",
    "CREATE INDEX idx_time ON log (time);",
    "CREATE INDEX idx_serial ON log (serial);",
];
const QUERY_INSERT: &'static str = "
INSERT INTO log (
  time,
  latitude,
  longitude,
  speed,
  height,
  direction,
  serial,
  is_satellite
)
VALUES (
  :time,
  :latitude,
  :longitude,
  :speed,
  :height,
  :direction,
  :serial,
  :is_satellite
)";

pub fn connectdb() -> Result<Connection> {
    let conn = Connection::open(DBPATH)?;
    Ok(conn)
}

pub fn initdb(conn: &Connection) -> Result<()> {
    create_tables(&conn)?;
    set_pragmas(&conn)?;
    Ok(())
}

fn create_tables(conn: &Connection) -> Result<()> {
    for s in STATEMENTS {
        match conn.execute(s, ()) {
            Ok(_) => {}
            Err(err) => println!("update failed: {}", err),
        }
    }
    Ok(())
}

fn set_pragmas(conn: &Connection) -> Result<()> {
    conn.pragma_update(Some(DatabaseName::Main), "journal_mode", "WAL")?;
    Ok(())
}

pub fn prepare_insert(conn: &Connection) -> Statement {
    conn.prepare(QUERY_INSERT).unwrap()
}

pub fn insert(conn: &Connection, dblog: &DbLog) -> Result<()> {
    let mut stmt = prepare_insert(&conn);
    match stmt.execute(params![
        dblog.time,
        dblog.latitude,
        dblog.longitude,
        dblog.speed,
        dblog.height,
        dblog.direction,
        dblog.serial,
        dblog.is_satellite,
    ]) {
        Ok(i) => println!("{} rows were inserted", i),
        Err(err) => println!("insert failed: {}", err),
    }
    Ok(())
}

pub struct DbLog {
    pub time: String,
    pub latitude: f64,
    pub longitude: f64,
    pub speed: u16,
    pub height: u16,
    pub direction: u16,
    pub serial: u16,
    pub is_satellite: bool,
}

impl ToSql for DbLog {
    fn to_sql(&self) -> Result<ToSqlOutput<'_>> {
        Ok(self.time.to_sql().unwrap())
    }
}
